package com.vp.plugin.sample.createinnerclass.actions;

import java.awt.event.*;

import com.vp.plugin.*;
import com.vp.plugin.action.*;
import com.vp.plugin.diagram.*;
import com.vp.plugin.model.*;
import com.vp.plugin.model.factory.*;

public class CreateInnerClassContextActionController implements VPContextActionController {
	
	/**
	 * Create InnerClass.  please refer to <a href="http://knowhow.visual-paradigm.com/?p=2378" target="_blank">http://knowhow.visual-paradigm.com/?p=2378</a>
	 */
	public void performAction(VPAction aAction, VPContext aContext, ActionEvent aE) {
		IModelElement lModel = aContext.getModelElement();
		if (lModel != null && lModel instanceof IClass) {
			// Create inner class based on this class
			IClass lClass = (IClass) lModel;
			IShapeUIModel lClassShape = (IShapeUIModel) aContext.getDiagramElement();
			IDiagramUIModel lDiagram = aContext.getDiagram();
			DiagramManager lDiagramManager = ApplicationManager.instance().getDiagramManager();
			
			// create inner class's model
			IClass lInnerClass = (IClass) lClass.createChild(IModelElementFactory.MODEL_TYPE_CLASS);
			// create inner class's shape
			IShapeUIModel lInnerClassShape = (IShapeUIModel) lDiagramManager.createDiagramElement(lDiagram, lInnerClass);
			// create containment
			lDiagramManager.createConnector(lDiagram, IClassDiagramUIModel.SHAPETYPE_CONTAINMENT, lClassShape, lInnerClassShape, null);
			
			lDiagram.layoutDiagramElement(new IDiagramElement[] {lInnerClassShape});
			return;
		}
		
		IDiagramUIModel lDiagram = aContext.getDiagram();
		if (lDiagram != null && lDiagram instanceof IClassDiagramUIModel) {
			// Create inner class on class diagram
			DiagramManager lDiagramManager = ApplicationManager.instance().getDiagramManager();
			
			// create class (create from diagram's parent model or create as root element)
			IClass lClass;
			if (lDiagram.getParentModel() != null) {
				lClass = (IClass) lDiagram.getParentModel().createChild(IModelElementFactory.MODEL_TYPE_CLASS);
				// Parent Model may not support creating child
				if (lClass == null) {
					lClass = IModelElementFactory.instance().createClass();
				}
			}
			else {
				lClass = IModelElementFactory.instance().createClass();
			}
			// create inner class
			IClass lInnerClass = (IClass) lClass.createChild(IModelElementFactory.MODEL_TYPE_CLASS);
			// create class + inner class's shape
			IShapeUIModel lClassShape = (IShapeUIModel) lDiagramManager.createDiagramElement(lDiagram, lClass);
			IShapeUIModel lInnerClassShape = (IShapeUIModel) lDiagramManager.createDiagramElement(lDiagram, lInnerClass);
			// create containment
			lDiagramManager.createConnector(lDiagram, IClassDiagramUIModel.SHAPETYPE_CONTAINMENT, lClassShape, lInnerClassShape, null);
			
			lDiagram.layoutDiagramElement(new IDiagramElement[] {lClassShape, lInnerClassShape});
			return;
		}
		
	}

	public void update(VPAction aAction, VPContext aContext) {
		IModelElement lModel = aContext.getModelElement();
		if (lModel != null && lModel instanceof IClass) {
			// Create inner class based on this class
			aAction.setEnabled(true);
			return;
		}
		
		IDiagramUIModel lDiagram = aContext.getDiagram();
		if (lDiagram != null && lDiagram instanceof IClassDiagramUIModel) {
			// Create inner class on class diagram
			aAction.setEnabled(true);
			return ;
		}
		
		aAction.setEnabled(false);
		
	}

}
